#include <bits/stdc++.h>
#define taskname "TREE"
using namespace std;

typedef long long ll;

class SegTree {
private:
    int sz;
    vector<ll> ST, freq, Lazy;

public:
    SegTree(int sz) {
        this->sz = sz;
        ST.assign(4 * sz + 1, 0);
        freq.assign(4 * sz + 1, 0);
        Lazy.assign(4 * sz + 1, 0);
        build(1, 1, sz);
    }

    void build(int id, int l, int r) {
        if (l == r) return void(freq[id] = 1);
        int m = (l + r) >> 1;
        build(id << 1, l, m);
        build(id << 1 | 1, m + 1, r);
        freq[id] = freq[id << 1] + freq[id << 1 | 1];
    }

    void push_down(int id, int l, int r) {
        ST[id] += Lazy[id] * freq[id];
        if (l < r) {
            Lazy[id << 1] += Lazy[id];
            Lazy[id << 1 | 1] += Lazy[id];
        }
        Lazy[id] = 0;
    }

    void update(int id, int l, int r, int u, int v, ll val) {
        push_down(id, l, r);
        if (r < u || v < l) return ;
        if (u <= l && r <= v) {
            Lazy[id] += val;
            return push_down(id, l, r);
        }
        int m = (l + r) >> 1;
        update(id << 1, l, m, u, v, val);
        update(id << 1 | 1, m + 1, r, u, v, val);

        ST[id] = ST[id << 1] + ST[id << 1 | 1];
    }

    void remove(int id, int l, int r, int p) {
        push_down(id, l, r);
        if (r < p || p < l) return ;
        if (l == r) {
            freq[id] = Lazy[id] = ST[id] = 0;
            return push_down(id, l, r);
        }
        int m = (l + r) >> 1;
        remove(id << 1, l, m, p);
        remove(id << 1 | 1, m + 1, r, p);

        ST[id] = ST[id << 1] + ST[id << 1 | 1];
        freq[id] = freq[id << 1] + freq[id << 1 | 1];
    }

    int get(int id, int l, int r, int p) {
        if (l == r) return l;
        int m = (l + r) >> 1;
        if (p <= freq[id << 1]) return get(id << 1, l, m, p);
        return get(id << 1 | 1, m + 1, r, p - freq[id << 1]);
    }

    ll get(int id, int l, int r, int u, int v) {
        push_down(id, l, r);
        if (r < u || v < l) return 0ll;
        if (u <= l && r <= v) return ST[id];
        int m = (l + r) >> 1;
        return get(id << 1, l, m, u, v) + get(id << 1 | 1, m + 1, r, u, v);
    } 
    
    void update(int l, int r, ll val) { 
        int pl = get(1, 1, sz, l);
        int pr = get(1, 1, sz, r);
        update(1, 1, sz, pl, pr, val); 
    }

    void remove(int p) {
        int _p = get(1, 1, sz, p);
        remove(1, 1, sz, _p);
    }

    ll get(int l, int r) {
        int pl = get(1, 1, sz, l);
        int pr = get(1, 1, sz, r);
        return get(1, 1, sz, pl, pr);
    }
};

void Input() {
    ios_base::sync_with_stdio(false);
    cin.tie(0); cout.tie(0);
    if (fopen(taskname".inp", "r")) {
        freopen(taskname".inp", "r", stdin);
        freopen(taskname".out", "w", stdout);
    }
}

void Solve() {
    int n, q; cin >> n >> q;
    SegTree ST(n);
    for (int i = 1; i <= n; i++) {
        ll x; cin >> x;
        ST.update(i, i, x);
    }

    while (q--) {
        int type; cin >> type;
        if (type == 1) {
            int p; cin >> p;
            ST.remove(p);
        }
        else if (type == 2) {
            int l, r, x; cin >> l >> r >> x;
            ST.update(l, r, x);
        }
        else {
            int l, r; cin >> l >> r;
            cout << ST.get(l, r) << "\n";
        }
    }
}

int main(){
    Input();
    Solve();
    return 0;
}